<?php

/**
 * The Keon Toolset hooks callback functionality of the plugin.
 *
 */
class Keon_Toolset_Hooks {

    private $hook_suffix;

    public static function instance() {

        static $instance = null;

        if ( null === $instance ) {
            $instance = new self();
        }

        return $instance;
    }

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     */
    public function __construct() {}

    /**
     * Check to see if advanced import plugin is not installed or activated.
     * Adds the Demo Import menu under Apperance.
     *
     * @since    1.0.0
     */
    public function import_menu() {
        if( !class_exists('Advanced_Import')){
            $this->hook_suffix[] = add_theme_page( esc_html__( 'Demo Import ','keon-toolset' ), esc_html__( 'Demo Import','keon-toolset'  ), 'manage_options', 'advanced-import', array( $this, 'demo_import_screen' ) );
        } 
    }

    /**
     * Enqueue styles.
     *
     * @since    1.0.0
     */
    public function enqueue_styles( $hook_suffix ) {
        if ( !is_array($this->hook_suffix) || !in_array( $hook_suffix, $this->hook_suffix )){
            return;
        }
        wp_enqueue_style( 'keon-toolset', KEON_TEMPLATE_URL . 'assets/keon-toolset.css',array( 'wp-admin', 'dashicons' ), '1.0.0', 'all' );
    }

    /**
     * Enqueue scripts.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts( $hook_suffix ) {
        if ( !is_array($this->hook_suffix) || !in_array( $hook_suffix, $this->hook_suffix )){
            return;
        }

        wp_enqueue_script( 'keon-toolset', KEON_TEMPLATE_URL . 'assets/keon-toolset.js', array( 'jquery'), '1.0.0', true );
        wp_localize_script( 'keon-toolset', 'keon_toolset', array(
            'btn_text' => esc_html__( 'Processing...', 'keon-toolset' ),
            'nonce'    => wp_create_nonce( 'keon_toolset_nonce' )
        ) );
    }

    /**
     * The demo import menu page comtent.
     *
     * @since    1.0.0
     */
    public function demo_import_screen() {
        ?>
        <div id="ads-notice">
            <div class="ads-container">
                <img class="ads-screenshot" src="<?php echo esc_url( keon_toolset_get_theme_screenshot() ) ?>" >
                <div class="ads-notice">
                    <h2>
                        <?php
                        printf(
                            esc_html__( 'Thank you for choosing %1$s! It is detected that an essential plugin, Advanced Import, is not activated. Importing demos for %1$s can begin after pressing the button below.', 'keon-toolset' ), '<strong>'. wp_get_theme()->get('Name'). '</strong>');
                        ?>
                    </h2>

                    <p class="plugin-install-notice"><?php esc_html_e( 'Clicking the button below will install and activate the Advanced Import plugin.', 'keon-toolset' ); ?></p>

                    <a class="ads-gsm-btn button" href="#" data-name="" data-slug="" aria-label="<?php esc_html_e( 'Get started with the Theme', 'keon-toolset' ); ?>">
                        <?php esc_html_e( 'Install Now', 'keon-toolset' );?>
                    </a>
                </div>
            </div>
        </div>
        <?php

    }

    /**
     * Installs or activates advanced import plugin if not detected as such.
     *
     * @since    1.0.0
     */
    public function install_advanced_import() {

        check_ajax_referer( 'keon_toolset_nonce', 'security' );

        $slug   = 'advanced-import';
        $plugin = 'advanced-import/advanced-import.php';
        $status = array(
            'install' => 'plugin',
            'slug'    => sanitize_key( wp_unslash( $slug ) ),
        );
        $status['redirect'] = admin_url( '/themes.php?page=advanced-import&browse=all&at-gsm-hide-notice=welcome' );

        if ( is_plugin_active_for_network( $plugin ) || is_plugin_active( $plugin ) ) {
            // Plugin is activated
            wp_send_json_success($status);
        }

        if ( ! current_user_can( 'install_plugins' ) ) {
            $status['errorMessage'] = __( 'Sorry, you are not allowed to install plugins on this site.', 'keon-toolset' );
            wp_send_json_error( $status );
        }

        include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        include_once ABSPATH . 'wp-admin/includes/plugin-install.php';

        // Looks like a plugin is installed, but not active.
        if ( file_exists( WP_PLUGIN_DIR . '/' . $slug ) ) {
            $plugin_data          = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin );
            $status['plugin']     = $plugin;
            $status['pluginName'] = $plugin_data['Name'];

            if ( current_user_can( 'activate_plugin', $plugin ) && is_plugin_inactive( $plugin ) ) {
                $result = activate_plugin( $plugin );

                if ( is_wp_error( $result ) ) {
                    $status['errorCode']    = $result->get_error_code();
                    $status['errorMessage'] = $result->get_error_message();
                    wp_send_json_error( $status );
                }

                wp_send_json_success( $status );
            }
        }

        $api = plugins_api(
            'plugin_information',
            array(
                'slug'   => sanitize_key( wp_unslash( $slug ) ),
                'fields' => array(
                    'sections' => false,
                ),
            )
        );

        if ( is_wp_error( $api ) ) {
            $status['errorMessage'] = $api->get_error_message();
            wp_send_json_error( $status );
        }

        $status['pluginName'] = $api->name;

        $skin     = new WP_Ajax_Upgrader_Skin();
        $upgrader = new Plugin_Upgrader( $skin );
        $result   = $upgrader->install( $api->download_link );

        if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
            $status['debug'] = $skin->get_upgrade_messages();
        }

        if ( is_wp_error( $result ) ) {
            $status['errorCode']    = $result->get_error_code();
            $status['errorMessage'] = $result->get_error_message();
            wp_send_json_error( $status );
        } elseif ( is_wp_error( $skin->result ) ) {
            $status['errorCode']    = $skin->result->get_error_code();
            $status['errorMessage'] = $skin->result->get_error_message();
            wp_send_json_error( $status );
        } elseif ( $skin->get_errors()->get_error_code() ) {
            $status['errorMessage'] = $skin->get_error_messages();
            wp_send_json_error( $status );
        } elseif ( is_null( $result ) ) {
            require_once( ABSPATH . 'wp-admin/includes/file.php' );
            WP_Filesystem();
            global $wp_filesystem;

            $status['errorCode']    = 'unable_to_connect_to_filesystem';
            $status['errorMessage'] = __( 'Unable to connect to the filesystem. Please confirm your credentials.', 'keon-toolset' );

            // Pass through the error from WP_Filesystem if one was raised.
            if ( $wp_filesystem instanceof WP_Filesystem_Base && is_wp_error( $wp_filesystem->errors ) && $wp_filesystem->errors->get_error_code() ) {
                $status['errorMessage'] = esc_html( $wp_filesystem->errors->get_error_message() );
            }

            wp_send_json_error( $status );
        }

        $install_status = install_plugin_install_status( $api );

        if ( current_user_can( 'activate_plugin', $install_status['file'] ) && is_plugin_inactive( $install_status['file'] ) ) {
            $result = activate_plugin( $install_status['file'] );

            if ( is_wp_error( $result ) ) {
                $status['errorCode']    = $result->get_error_code();
                $status['errorMessage'] = $result->get_error_message();
                wp_send_json_error( $status );
            }
        }

        wp_send_json_success( $status );

    }

    /**
     * Demo list of the Keon Themes with their recommended plugins.
     *
     * @since    1.0.0
     */
    public function keon_toolset_demo_import_lists($list){

        $theme_slug = keon_toolset_get_theme_slug();
        $demo_lists = array(
                    array(
                    'title' => __( 'Agency', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'agency', 'multipurpose' ),
                    'categories' => array( 'agency' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/agency/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/agency/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/agency/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo8/gutener-agency-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo8/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 1', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 1', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog1/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog1/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog1/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo2/gutener-blog-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo2/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 2', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 2', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog2/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog2/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog2/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo201/gutener-blog-2-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo201/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 3', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 3', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog3/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog3/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog3/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo202/gutener-blog-3-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo202/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 4', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 4', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog4/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog4/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog4/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo203/gutener-blog4-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo203/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 5', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 5', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog5/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog5/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog5/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo204/gutener-blog5-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo204/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 6', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 6', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog6/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog6/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog6/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo205/gutener-blog6-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo205/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 7', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 7', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog7/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog7/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog7/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo206/gutener-blog7-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo206/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 8', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 8', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog8/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog8/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog8/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo208/gutener-blog8-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo208/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 9', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 9', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog9/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog9/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog9/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo209/gutener-blog9-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo209/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Blog 10', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'blog', 'blog 10', 'multipurpose' ),
                    'categories' => array( 'blog' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/blog/blog10/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/blog/blog10/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/blog/blog10/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo7/gutener-blog10-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo7/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Business 1', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'business', 'business 1', 'multipurpose' ),
                    'categories' => array( 'business' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/business/business1/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/business/business1/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/business/business1/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo102/gutener-business-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo102/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Business 2', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'business', 'business 2', 'multipurpose' ),
                    'categories' => array( 'business' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/business/business2/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/business/business2/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/business/business2/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo108/gutener-business2-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo108/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Car Dealer', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'car', 'dealer', 'multipurpose' ),
                    'categories' => array( 'car-dealer' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/car-dealer/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/car-dealer/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/car-dealer/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo5/gutener-car-dealer-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo5/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Charity NGO 1', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'charity', 'ngo', 'charity 1', 'ngo 1', 'multipurpose' ),
                    'categories' => array( 'charity' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/charity-ngo/charity-ngo1/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/charity-ngo/charity-ngo1/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/charity-ngo/charity-ngo1/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo9/gutener-charity-NGO1-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo9/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Charity NGO 2', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'charity', 'ngo', 'charity 2', 'ngo 2', 'multipurpose' ),
                    'categories' => array( 'charity' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/charity-ngo/charity-ngo2/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/charity-ngo/charity-ngo2/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/charity-ngo/charity-ngo2/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo109/gutener-charity-NGO-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo109/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Coming Soon', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'coming', 'soon', 'coming soon', 'multipurpose' ),
                    'categories' => array( 'coming-soon' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/coming-soon/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/coming-soon/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/coming-soon/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo207/gutener-coming-soon-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo207/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Construction', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'construction', 'multipurpose' ),
                    'categories' => array( 'construction' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/construction/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/construction/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/construction/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo105/gutener-construction-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo105/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Consultancy', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'consultancy', 'multipurpose' ),
                    'categories' => array( 'consultancy' ),
                    'template_url' => array(
                          'content' => KEON_TOOLSET_URL.'gutener/consultancy/content.json',
                          'options' => KEON_TOOLSET_URL.'gutener/consultancy/options.json',
                          'widgets' => KEON_TOOLSET_URL.'gutener/consultancy/widgets.json'
                    ),
                   'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo1/gutener-consultancy-760-460.jpg',
                   'demo_url' => 'https://demo.keonthemes.com/gutener/demo1/',
                   /* Recommended plugin for this demo */
                   'plugins' => array(
                        array(
                         'name' => __( 'Contact Form 7', 'keon-toolset' ),
                         'slug' => 'contact-form-7',
                         'main_file' => 'wp-contact-form-7.php',
                        ),
                        array(
                         'name' => __( 'Gutentor', 'keon-toolset' ),
                         'slug' => 'gutentor',
                        ),
                        array(
                         'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                         'slug' => 'mailchimp-for-wp',
                        ),
                        array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Corporate 1', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'corporate', 'corporate 1', 'multipurpose' ),
                    'categories' => array( 'corporate' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/corporate/corporate1/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/corporate/corporate1/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/corporate/corporate1/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo107/gutener-corporate-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo107/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Corporate 2', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'corporate', 'corporate 2', 'multipurpose' ),
                    'categories' => array( 'corporate' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/corporate/corporate2/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/corporate/corporate2/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/corporate/corporate2/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo6/gutener-corporate2-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo6/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'eCommerce', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'ecommerce', 'eCommerce', 'multipurpose' ),
                    'categories' => array( 'ecommerce' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/ecommerce/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/ecommerce/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/ecommerce/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo110/gutener-ecommerce-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo110/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                     array(
                         'name' => __( 'Woo Commerce', 'keon-toolset' ),
                         'slug' => 'woocommerce',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Education', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'education', 'multipurpose' ),
                    'categories' => array( 'education' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/education/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/education/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/education/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo104/gutener-education-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo104/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Event 1', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'event 1', 'event', 'multipurpose' ),
                    'categories' => array( 'event' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/event/event1/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/event/event1/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/event/event1/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo113/gutener-event-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo113/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Event 2', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'event 2', 'event', 'multipurpose' ),
                    'categories' => array( 'event' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/event/event2/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/event/event2/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/event/event2/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo114/gutener-event2-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo114/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Finance', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'finance', 'multipurpose' ),
                    'categories' => array( 'finance' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/finance/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/finance/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/finance/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo101/gutener-finance-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo101/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Fitness', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'fitness', 'multipurpose' ),
                    'categories' => array( 'fitness' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/fitness/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/fitness/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/fitness/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo4/gutener-fitness-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo4/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Magazine', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'magazine', 'multipurpose' ),
                    'categories' => array( 'magazine' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/magazine/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/magazine/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/magazine/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo106/gutener-magazine-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo106/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Marketing', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'marketing', 'multipurpose' ),
                    'categories' => array( 'marketing' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/marketing/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/marketing/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/marketing/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo3/gutener-marketing-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo3/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Medical', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'medical', 'multipurpose' ),
                    'categories' => array( 'medical' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/medical/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/medical/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/medical/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo103/gutener-medical-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo103/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Music', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'music', 'multipurpose' ),
                    'categories' => array( 'music' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/music/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/music/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/music/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo111/gutener-music-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo111/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Online Courses 1', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'online 1', 'courses 1', 'online courses 1', 'online', 'courses', 'online courses', 'multipurpose' ),
                    'categories' => array( 'online-courses' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/online-courses/online-courses1/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/online-courses/online-courses1/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/online-courses/online-courses1/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo10/gutener-online-course-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo10/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
                    array(
                    'title' => __( 'Online Courses 2', 'keon-toolset' ),
                    'is_pro' => true,
                    'pro_url' => 'https://keonthemes.com/downloads/gutener-pro',
                    'type' => 'normal',
                    'author' => __( 'Keon Themes', 'keon-toolset' ),
                    'keywords' => array( 'online 2', 'courses 2', 'online courses 2', 'online', 'courses', 'online courses', 'multipurpose' ),
                    'categories' => array( 'online-courses' ),
                    'template_url' => array(
                       'content' => KEON_TOOLSET_URL.'gutener/online-courses/online-courses2/content.json',
                       'options' => KEON_TOOLSET_URL.'gutener/online-courses/online-courses2/options.json',
                       'widgets' => KEON_TOOLSET_URL.'gutener/online-courses/online-courses2/widgets.json'
                    ),
                    'screenshot_url' => 'https://demo.keonthemes.com/gutener/screenshots/demo112/gutener-online-course2-760-460.jpg',
                    'demo_url' => 'https://demo.keonthemes.com/gutener/demo112/',
                    /* Recommended plugin for this demo */
                    'plugins' => array(
                     array(
                      'name' => __( 'Contact Form 7', 'keon-toolset' ),
                      'slug' => 'contact-form-7',
                      'main_file' => 'wp-contact-form-7.php',
                     ),
                     array(
                      'name' => __( 'Gutentor', 'keon-toolset' ),
                      'slug' => 'gutentor',
                     ),
                     array(
                      'name' => __( 'Mailchimp for WordPress', 'keon-toolset' ),
                      'slug' => 'mailchimp-for-wp',
                     ),
                     array(
                         'name' => __( 'AddToAny Share Buttons', 'keon-toolset' ),
                         'slug' => 'add-to-any',
                        ),
                    )
                    ),
        );
            
        foreach ($demo_lists as &$val){
            switch( $theme_slug ):
                case 'gutener-pro':
                case 'gutener-pro-child':
                    if($val['title']=='Agency'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 1'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 2'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 3'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 4'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 5'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 6'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 7'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 8'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 9'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 10'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Business 1'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Business 2'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Car Dealer'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Charity NGO 1'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Charity NGO 2'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Coming Soon'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Construction'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Consultancy'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Corporate 1'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Corporate 2'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='eCommerce'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Education'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Event 1'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Event 2'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Fitness'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Finance'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Magazine'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Marketing'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Medical'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Music'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Online Courses 1'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Online Courses 2'){
                        $val['is_pro']=false;
                    }
                    break;
                case 'gutener':
                case 'gutener-charity-ngo':
                    if($val['title']=='Agency'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 1'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 2'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 4'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 6'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 7'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 8'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 9'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Blog 10'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Charity NGO 1'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Consultancy'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Corporate 1'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='eCommerce'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Fitness'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Marketing'){
                        $val['is_pro']=false;
                    }elseif($val['title']=='Online Courses 1'){
                        $val['is_pro']=false;
                    }
                    break;
                default:
                $demo_lists = array();
            endswitch;
        }
   return array_merge($list, $demo_lists);
}
}

/**
 * Begins execution of the hooks.
 *
 * @since    1.0.0
 */
function keon_toolset_hooks( ) {
    return Keon_Toolset_Hooks::instance();
}





